function [mu,TMN]=eigTMNsd(alpha,beta,b,M,N)
% eigTMNsd solution operator approach for the time-varying DDE
%       v'(t) = alpha*v(t)+beta*v(t-tau(t))-beta*gamma(t)*v(t-b),
% which is the linarization around a bounded periodic solution of the
% state-dependent DDE
%       u'(t) = alpha*u(t)+beta*u(t-1-u(t-b)).
% [mu,TMN]=eigTMNsd(alpha,beta,b,M,N) returns in output the approximated
% eigenvalues mu of the evolution operator T(omega,0) and its discretized
% matrix TMN.

%% Import of the periodic solution and computation of tau and gamma
% The file .txt contains the periodic solution. Its last component contains
% the period omega. To correctly reconstruct the periodic solution, we copy
% in the last component of the vector sol, the first value of the periodic
% solution and use griddedInterpolant to perform the interpolation on the
% points pti. Then we define tau and gamma.
d=1;                                                           %dimension 1
Id=eye(d);
data=importdata('HKRSb0_sd500_7.txt');
omega=data(end);
sol=data;
sol(end)=sol(1);
h=omega/(length(data)-1);
pti=linspace(0,omega,length(data));
F=griddedInterpolant(pti,sol);
xbar=@(t)F(mod(t,omega));
tau=@(t)1+xbar(t-b);
gamma=@(t)alpha*xbar(t-1-xbar(t-b))+...
    beta*xbar(t-2-xbar(t-b)-xbar(t-1-xbar(t-b)-b));

%% MESH OmegaN IN [0,omega] (ordered from left=0 to right=omega)
zN=cos((2*(1:N)-1)*pi/(2*N));   %Chebyshev I nodes in [-1,1]
OmegaN=omega*(1-zN)/2;          %nodes in [0,omega]
wbN=barywei(OmegaN);            %Chebyshev I barycentric weights in [0,omega]

%% Determination of the maximum delay TAU
% We compute tau for the points of the mesh OmegaN and some other auxiliary
% points between them and determine the maximum delay TAU
tauaux=zeros(N+101,1);
for i=1:N
    tauaux(i)=tau(OmegaN(i));
end
tauaux(N+1)=b;
ptiaux=linspace(0,omega,100);
for i=1:100
    tauaux(N+1+i)=tau(ptiaux(i));
end
TAU=max(tauaux);

%% MESH OmegaM IN [-TAU,0] (ordered from right=0 to left=-TAU)
zM=cos((0:M)*pi/M);           %Chebyshev II nodes in [-1,1]
OmegaM=TAU*(zM-1)/2;          %nodes in [-TAU,0]
[wbM]=barywei(OmegaM);        %Chebyshev II barycentric weights in [-TAU,0]

%% MATRIX TM1
TM=[ones(M+1,1),zeros(M+1,M)];
TM1=kron(TM,Id);                               %block structure for systems

%% MATRIX TMN2
TMN2=zeros(M+1,N);                      %initialization
for m=0:M                               %along rows
    bbb=omega+OmegaM(m+1);              %integral upper extremum
    for i=1:N                           %along columns
        lagvali=@(t)arrayfun(@(tt)(prod((tt-OmegaN([1:i-1, i+1:end])))*wbN(i))*...
            (tt~=OmegaN(i))+1*(tt==OmegaN(i)),t);
        %1st barycentric formula
        TMN2(m+1,i)=integral(lagvali,0,bbb);
        %element of the matrix TMN2
    end
end
%% MATRIX UMN1
UMN1=zeros(d*N,d*(M+1));              %initialization
for n=1:N                             %along rows
    if OmegaN(n)>tau(OmegaN(n))       %condition for tau
        if OmegaN(n)>b                %condition for the constant delay
            for m=0:M                 %along columns
                lagval0=(prod((0-OmegaM([1:m, m+2:end])))*wbM(m+1))*...
                    (0~=OmegaM(m+1))+1*(0==OmegaM(m+1));
                %1st barycentric formula (m=0)
                UMN1(n,m+1)=alpha*lagval0+beta*lagval0-...
                    beta*gamma(OmegaN(n))*lagval0;
                %element of the matrix UMN1
            end
        else
            for m=0:M                 %along columns
                tnb=OmegaN(n)-b;      %point considered
                lagval0=(prod((0-OmegaM([1:m, m+2:end])))*wbM(m+1))*...
                    (0~=OmegaM(m+1))+1*(0==OmegaM(m+1));
                %1st barycentric formula (m=0)
                lagvalm=(prod((tnb-OmegaM([1:m, m+2:end])))*wbM(m+1))*...
                    (tnb~=OmegaM(m+1))+1*(tnb==OmegaM(m+1));
                %1st barycentric formula
                UMN1(n,m+1)=alpha*lagval0+beta*lagval0-...
                    beta*gamma(OmegaN(n))*lagvalm;
                %element of the matrix UMN1
            end
        end
    else
        for m=0:M                     %along columns
            lagval0=(prod((0-OmegaM([1:m, m+2:end])))*wbM(m+1))*...
                (0~=OmegaM(m+1))+1*(0==OmegaM(m+1));
            tn_tau_tn=OmegaN(n)-tau(OmegaN(n));
            %point considered
            lagvalm=(prod((tn_tau_tn-OmegaM([1:m, m+2:end])))*wbM(m+1))*...
                (tn_tau_tn~=OmegaM(m+1))+1*(tn_tau_tn==OmegaM(m+1));
            %1st barycentric formula
            UMN1(n,m+1)=alpha*lagval0+beta*lagvalm;
            %element of the matrix UMN1
            if OmegaN(n)>b            %condition for the constant delay
                UMN1(n,m+1)=UMN1(n,m+1)-beta*gamma(OmegaN(n))*lagval0;
                %update of the element of the matrix UMN1
            else
                tnb=OmegaN(n)-b;
                lagvalm=(prod((tnb-OmegaM([1:m,m+2:end])))*wbM(m+1))*...
                    (tnb~=OmegaM(m+1))+1*(tnb==OmegaM(m+1));
                %1st barycentric formula
                UMN1(n,m+1)=UMN1(n,m+1)-beta*gamma(OmegaN(n))*lagvalm;
                %update of the element of the matrix UMN1
            end
        end
    end
end
%% MATRIX UN2
UN2=zeros(d*N);
for n=1:N                                  %along rows
    tn=OmegaN(n);                          %point considered
    if tn>tau(tn)                          %condition for tau
        for i=1:N                          %along columns
            lagvali=@(t)arrayfun(@(tt)(prod((tt-OmegaN([1:i-1,i+1:end])))*wbN(i))*...
                (tt~=OmegaN(i))+1*(tt==OmegaN(i)),t);
            %1st barycentric formula
            UN2(n,i)=alpha*integral(lagvali,0,tn)+beta*integral(lagvali,0,tn-tau(tn));
            %element of matrix UN2
            if tn>b                       %condition for the constant delay
                UN2(n,i)=UN2(n,i)-beta*gamma(tn)*integral(lagvali,0,tn-b);
                %update of the element of matrix UN2
            end
        end
    else
        for i=1:N                         %along columns
            lagvali=@(t)arrayfun(@(tt)(prod((tt-OmegaN([1:i-1,i+1:end])))*wbN(i))*...
                (tt~=OmegaN(i))+1*(tt==OmegaN(i)),t);
            %1st barycentric formula
            UN2(n,i)=alpha*integral(lagvali,0,tn);
            %element of the matrix UN2
            if tn>b                       %condition for the constant delay
                UN2(n,i)=UN2(n,i)-beta*gamma(tn)*integral(lagvali,0,tn-b);
                %update of the element of matrix UN2
            end
        end
    end
end
%% MATRIX TMN
TMN=TM1+TMN2*((eye(d*N)-UN2)\UMN1); %evolution operator final discretization

%% COMPUTATION OF EIGENVALUES
mu=eig(TMN);                  %approximated eigenvalues
mu=sort(mu,'descend');        %ordered by decreasing modulus
%% AUXILIARY FUNCTIONS

function wb=barywei(x)
%WEIGHTS barycentric weights.
%  wb=BARYWEI(x) returns the weights wb for Lagrange barycentric
%  interpolation for the nodes x

N=length(x)-1;
dx=ones(N+1,1);
wb=dx;
for m=2:N+1
    for i=1:m-1
        dx(i)=x(m)-x(i);
        wb(i)=-dx(i)*wb(i);
    end
    wb(m)=prod(dx(1:m));
end
wb=1./wb';